@@ -30,14 +30,15 @@ modules = [
     "batoto",
     "bbc",
     "behance",
+    "bilibili",
     "blogger",
     "bluesky",
+    "boosty",
     "bunkr",
     "catbox",
     "chevereto",
     "cien",
     "civitai",
-    "cohost",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -46,9 +47,10 @@ modules = [
     "dynastyscans",
     "e621",
     "erome",
+    "everia",
     "exhentai",
+    "facebook",
     "fanbox",
-    "fanleaks",
     "fantia",
     "fapello",
     "fapachi",
@@ -94,6 +96,7 @@ modules = [
     "lexica",
     "lightroom",
     "livedoor",
+    "lofter",
     "luscious",
     "lynxchan",
     "mangadex",
@@ -106,10 +109,12 @@ modules = [
     "mangasee",
     "mangoxo",
     "misskey",
+    "motherless",
     "myhentaigallery",
     "myportfolio",
     "naver",
     "naverwebtoon",
+    "nekohouse",
     "newgrounds",
     "nhentai",
     "nijie",
@@ -118,6 +123,7 @@ modules = [
     "nsfwalbum",
     "paheal",
     "patreon",
+    "pexels",
     "philomena",
     "photovogue",
     "picarto",
@@ -133,14 +139,18 @@ modules = [
     "pornhub",
     "pornpics",
     "postmill",
-    "pururin",
     "reactor",
     "readcomiconline",
+    "realbooru",
     "reddit",
     "redgifs",
     "rule34us",
+    "rule34vault",
+    "rule34xyz",
+    "saint",
     "sankaku",
     "sankakucomplex",
+    "scrolller",
     "seiga",
     "senmanga",
     "sexcom",
@@ -180,12 +190,15 @@ modules = [
     "weasyl",
     "webmshare",
     "webtoons",
+    "weebcentral",
     "weibo",
     "wikiart",
     "wikifeet",
     "wikimedia",
+    "xfolio",
     "xhamster",
     "xvideos",
+    "yiffverse",
     "zerochan",
     "zzup",
     "booru",
@@ -199,6 +212,7 @@ modules = [
     "directlink",
     "recursive",
     "oauth",
+    "noop",
     "ytdl",
     "generic",
 ]
