@@ -20,10 +20,10 @@ class DanbooruExtractor(BaseExtractor):
     page_limit = 1000
     page_start = None
     per_page = 200
-    request_interval = 1.0
+    useragent = util.USERAGENT
+    request_interval = (0.5, 1.5)
 
-    def __init__(self, match):
-        BaseExtractor.__init__(self, match)
+    def _init(self):
         self.ugoira = self.config("ugoira", False)
         self.external = self.config("external", False)
         self.includes = False
@@ -32,12 +32,12 @@ class DanbooruExtractor(BaseExtractor):
         if isinstance(threshold, int):
             self.threshold = 1 if threshold < 1 else threshold
         else:
-            self.threshold = self.per_page
+            self.threshold = self.per_page - 20
 
         username, api_key = self._get_auth_info()
         if username:
             self.log.debug("Using HTTP Basic Auth for user '%s'", username)
-            self.session.auth = (username, api_key)
+            self.session.auth = util.HTTPBasicAuth(username, api_key)
 
     def skip(self, num):
         pages = num // self.per_page
@@ -47,8 +47,8 @@ class DanbooruExtractor(BaseExtractor):
         return pages * self.per_page
 
     def items(self):
-        self.session.headers["User-Agent"] = util.USERAGENT
-
+        # 'includes' initialization must be done here and not in '_init()'
+        # or it'll cause an exception with e621 when 'metadata' is enabled
         includes = self.config("metadata")
         if includes:
             if isinstance(includes, (list, tuple)):
@@ -70,10 +70,33 @@ class DanbooruExtractor(BaseExtractor):
                 continue
 
             text.nameext_from_url(url, post)
+            post["date"] = text.parse_datetime(
+                post["created_at"], "%Y-%m-%dT%H:%M:%S.%f%z")
+
+            post["tags"] = (
+                post["tag_string"].split(" ")
+                if post["tag_string"] else ())
+            post["tags_artist"] = (
+                post["tag_string_artist"].split(" ")
+                if post["tag_string_artist"] else ())
+            post["tags_character"] = (
+                post["tag_string_character"].split(" ")
+                if post["tag_string_character"] else ())
+            post["tags_copyright"] = (
+                post["tag_string_copyright"].split(" ")
+                if post["tag_string_copyright"] else ())
+            post["tags_general"] = (
+                post["tag_string_general"].split(" ")
+                if post["tag_string_general"] else ())
+            post["tags_meta"] = (
+                post["tag_string_meta"].split(" ")
+                if post["tag_string_meta"] else ())
 
             if post["extension"] == "zip":
                 if self.ugoira:
-                    post["frames"] = self._ugoira_frames(post)
+                    post["_ugoira_original"] = False
+                    post["_ugoira_frame_data"] = post["frames"] = \
+                        self._ugoira_frames(post)
                     post["_http_adjust_extension"] = False
                 else:
                     url = post["large_file_url"]
@@ -86,48 +109,60 @@ class DanbooruExtractor(BaseExtractor):
             yield Message.Directory, post
             yield Message.Url, url, post
 
+    def items_artists(self):
+        for artist in self.artists():
+            artist["_extractor"] = DanbooruTagExtractor
+            url = "{}/posts?tags={}".format(
+                self.root, text.quote(artist["name"]))
+            yield Message.Queue, url, artist
+
     def metadata(self):
         return ()
 
     def posts(self):
         return ()
 
-    def _pagination(self, endpoint, params, pages=False):
+    def _pagination(self, endpoint, params, prefix=None):
         url = self.root + endpoint
         params["limit"] = self.per_page
         params["page"] = self.page_start
 
+        first = True
         while True:
             posts = self.request(url, params=params).json()
-            if "posts" in posts:
+            if isinstance(posts, dict):
                 posts = posts["posts"]
 
-            if self.includes and posts:
-                if not pages and "only" not in params:
-                    params["page"] = "b{}".format(posts[0]["id"] + 1)
-                params["only"] = self.includes
-                data = {
-                    meta["id"]: meta
-                    for meta in self.request(url, params=params).json()
-                }
-                for post in posts:
-                    post.update(data[post["id"]])
-                params["only"] = None
+            if posts:
+                if self.includes:
+                    params_meta = {
+                        "only" : self.includes,
+                        "limit": len(posts),
+                        "tags" : "id:" + ",".join(str(p["id"]) for p in posts),
+                    }
+                    data = {
+                        meta["id"]: meta
+                        for meta in self.request(
+                            url, params=params_meta).json()
+                    }
+                    for post in posts:
+                        post.update(data[post["id"]])
 
-            yield from posts
+                if prefix == "a" and not first:
+                    posts.reverse()
+
+                yield from posts
 
             if len(posts) < self.threshold:
                 return
 
-            if pages:
+            if prefix:
+                params["page"] = "{}{}".format(prefix, posts[-1]["id"])
+            elif params["page"]:
                 params["page"] += 1
             else:
-                for post in reversed(posts):
-                    if "id" in post:
-                        params["page"] = "b{}".format(post["id"])
-                        break
-                else:
-                    return
+                params["page"] = 2
+            first = False
 
     def _ugoira_frames(self, post):
         data = self.request("{}/posts/{}.json?only=media_metadata".format(
@@ -144,7 +179,8 @@ class DanbooruExtractor(BaseExtractor):
 BASE_PATTERN = DanbooruExtractor.update({
     "danbooru": {
         "root": None,
-        "pattern": r"(?:danbooru|hijiribe|sonohara|safebooru)\.donmai\.us",
+        "pattern": r"(?:(?:danbooru|hijiribe|sonohara|safebooru)\.donmai\.us"
+                   r"|donmai\.moe)",
     },
     "atfbooru": {
         "root": "https://booru.allthefallen.moe",
@@ -152,8 +188,12 @@ BASE_PATTERN = DanbooruExtractor.update({
     },
     "aibooru": {
         "root": None,
-        "pattern": r"(?:safe.)?aibooru\.online",
-    }
+        "pattern": r"(?:safe\.)?aibooru\.online",
+    },
+    "booruvar": {
+        "root": "https://booru.borvar.art",
+        "pattern": r"booru\.borvar\.art",
+    },
 })
 
 
@@ -163,33 +203,7 @@ class DanbooruTagExtractor(DanbooruExtractor):
     directory_fmt = ("{category}", "{search_tags}")
     archive_fmt = "t_{search_tags}_{id}"
     pattern = BASE_PATTERN + r"/posts\?(?:[^&#]*&)*tags=([^&#]*)"
-    test = (
-        ("https://danbooru.donmai.us/posts?tags=bonocho", {
-            "content": "b196fb9f1668109d7774a0a82efea3ffdda07746",
-        }),
-        # test page transitions
-        ("https://danbooru.donmai.us/posts?tags=mushishi", {
-            "count": ">= 300",
-        }),
-        # 'external' option (#1747)
-        ("https://danbooru.donmai.us/posts?tags=pixiv_id%3A1476533", {
-            "options": (("external", True),),
-            "pattern": r"https://i\.pximg\.net/img-original/img"
-                       r"/2008/08/28/02/35/48/1476533_p0\.jpg",
-        }),
-        ("https://booru.allthefallen.moe/posts?tags=yume_shokunin", {
-            "count": 12,
-        }),
-        ("https://aibooru.online/posts?tags=center_frills&z=1", {
-            "pattern": r"https://aibooru\.online/data/original"
-                       r"/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{32}\.\w+",
-            "count": ">= 3",
-        }),
-        ("https://hijiribe.donmai.us/posts?tags=bonocho"),
-        ("https://sonohara.donmai.us/posts?tags=bonocho"),
-        ("https://safebooru.donmai.us/posts?tags=bonocho"),
-        ("https://safe.aibooru.online/posts?tags=center_frills"),
-    )
+    example = "https://danbooru.donmai.us/posts?tags=TAG"
 
     def __init__(self, match):
         DanbooruExtractor.__init__(self, match)
@@ -200,7 +214,21 @@ class DanbooruTagExtractor(DanbooruExtractor):
         return {"search_tags": self.tags}
 
     def posts(self):
-        return self._pagination("/posts.json", {"tags": self.tags})
+        prefix = "b"
+        for tag in self.tags.split():
+            if tag.startswith("order:"):
+                if tag == "order:id" or tag == "order:id_asc":
+                    prefix = "a"
+                elif tag == "order:id_desc":
+                    prefix = "b"
+                else:
+                    prefix = None
+            elif tag.startswith(
+                    ("id:", "md5:", "ordfav:", "ordfavgroup:", "ordpool:")):
+                prefix = None
+                break
+
+        return self._pagination("/posts.json", {"tags": self.tags}, prefix)
 
 
 class DanbooruPoolExtractor(DanbooruExtractor):
@@ -209,17 +237,7 @@ class DanbooruPoolExtractor(DanbooruExtractor):
     directory_fmt = ("{category}", "pool", "{pool[id]} {pool[name]}")
     archive_fmt = "p_{pool[id]}_{id}"
     pattern = BASE_PATTERN + r"/pool(?:s|/show)/(\d+)"
-    test = (
-        ("https://danbooru.donmai.us/pools/7659", {
-            "content": "b16bab12bea5f7ea9e0a836bf8045f280e113d99",
-        }),
-        ("https://booru.allthefallen.moe/pools/9", {
-            "url": "902549ffcdb00fe033c3f63e12bc3cb95c5fd8d5",
-            "count": 6,
-        }),
-        ("https://aibooru.online/pools/1"),
-        ("https://danbooru.donmai.us/pool/show/7659"),
-    )
+    example = "https://danbooru.donmai.us/pools/12345"
 
     def __init__(self, match):
         DanbooruExtractor.__init__(self, match)
@@ -234,7 +252,7 @@ class DanbooruPoolExtractor(DanbooruExtractor):
 
     def posts(self):
         params = {"tags": "pool:" + self.pool_id}
-        return self._pagination("/posts.json", params)
+        return self._pagination("/posts.json", params, "b")
 
 
 class DanbooruPostExtractor(DanbooruExtractor):
@@ -242,22 +260,7 @@ class DanbooruPostExtractor(DanbooruExtractor):
     subcategory = "post"
     archive_fmt = "{id}"
     pattern = BASE_PATTERN + r"/post(?:s|/show)/(\d+)"
-    test = (
-        ("https://danbooru.donmai.us/posts/294929", {
-            "content": "5e255713cbf0a8e0801dc423563c34d896bb9229",
-        }),
-        ("https://danbooru.donmai.us/posts/3613024", {
-            "pattern": r"https?://.+\.zip$",
-            "options": (("ugoira", True),)
-        }),
-        ("https://booru.allthefallen.moe/posts/22", {
-            "content": "21dda68e1d7e0a554078e62923f537d8e895cac8",
-        }),
-        ("https://aibooru.online/posts/1", {
-            "content": "54d548743cd67799a62c77cbae97cfa0fec1b7e9",
-        }),
-        ("https://danbooru.donmai.us/post/show/294929"),
-    )
+    example = "https://danbooru.donmai.us/posts/12345"
 
     def __init__(self, match):
         DanbooruExtractor.__init__(self, match)
@@ -278,16 +281,7 @@ class DanbooruPopularExtractor(DanbooruExtractor):
     directory_fmt = ("{category}", "popular", "{scale}", "{date}")
     archive_fmt = "P_{scale[0]}_{date}_{id}"
     pattern = BASE_PATTERN + r"/(?:explore/posts/)?popular(?:\?([^#]*))?"
-    test = (
-        ("https://danbooru.donmai.us/explore/posts/popular"),
-        (("https://danbooru.donmai.us/explore/posts/popular"
-          "?date=2013-06-06&scale=week"), {
-            "range": "1-120",
-            "count": 120,
-        }),
-        ("https://booru.allthefallen.moe/explore/posts/popular"),
-        ("https://aibooru.online/explore/posts/popular"),
-    )
+    example = "https://danbooru.donmai.us/explore/posts/popular"
 
     def __init__(self, match):
         DanbooruExtractor.__init__(self, match)
@@ -307,7 +301,40 @@ class DanbooruPopularExtractor(DanbooruExtractor):
         return {"date": date, "scale": scale}
 
     def posts(self):
-        if self.page_start is None:
-            self.page_start = 1
-        return self._pagination(
-            "/explore/posts/popular.json", self.params, True)
+        return self._pagination("/explore/posts/popular.json", self.params)
+
+
+class DanbooruArtistExtractor(DanbooruExtractor):
+    """Extractor for danbooru artists"""
+    subcategory = "artist"
+    pattern = BASE_PATTERN + r"/artists/(\d+)"
+    example = "https://danbooru.donmai.us/artists/12345"
+
+    items = DanbooruExtractor.items_artists
+
+    def artists(self):
+        url = "{}/artists/{}.json".format(self.root, self.groups[-1])
+        return (self.request(url).json(),)
+
+
+class DanbooruArtistSearchExtractor(DanbooruExtractor):
+    """Extractor for danbooru artist searches"""
+    subcategory = "artist-search"
+    pattern = BASE_PATTERN + r"/artists/?\?([^#]+)"
+    example = "https://danbooru.donmai.us/artists?QUERY"
+
+    items = DanbooruExtractor.items_artists
+
+    def artists(self):
+        url = self.root + "/artists.json"
+        params = text.parse_query(self.groups[-1])
+        params["page"] = text.parse_int(params.get("page"), 1)
+
+        while True:
+            artists = self.request(url, params=params).json()
+
+            yield from artists
+
+            if len(artists) < 20:
+                return
+            params["page"] += 1
