@@ -124,6 +124,11 @@ class MoebooruPoolExtractor(MoebooruExtractor):
         self.pool_id = match.group(match.lastindex)
 
     def metadata(self):
+        if self.config("metadata"):
+            url = "{}/pool/show/{}.json".format(self.root, self.pool_id)
+            pool = self.request(url).json()
+            pool.pop("posts", None)
+            return {"pool": pool}
         return {"pool": text.parse_int(self.pool_id)}
 
     def posts(self):
