@@ -26,12 +26,22 @@ class PixivExtractor(Extractor):
     directory_fmt = ("{category}", "{user[id]} {user[account]}")
     filename_fmt = "{id}_p{num}.{extension}"
     archive_fmt = "{id}{suffix}.{extension}"
-    cookies_domain = None
+    cookies_domain = ".pixiv.net"
+    limit_url = "https://s.pximg.net/common/images/limit_"
+    # https://s.pximg.net/common/images/limit_sanity_level_360.png
+    # https://s.pximg.net/common/images/limit_unviewable_360.png
+    # https://s.pximg.net/common/images/limit_mypixiv_360.png
 
     def _init(self):
         self.api = PixivAppAPI(self)
         self.load_ugoira = self.config("ugoira", True)
+        self.load_ugoira_original = (self.load_ugoira == "original")
         self.max_posts = self.config("max-posts", 0)
+        self.sanity_workaround = self.config("sanity", True)
+        self.meta_user = self.config("metadata")
+        self.meta_bookmark = self.config("metadata-bookmark")
+        self.meta_comments = self.config("comments")
+        self.meta_captions = self.config("captions")
 
     def items(self):
         tags = self.config("tags", "japanese")
@@ -46,11 +56,7 @@ class PixivExtractor(Extractor):
             def transform_tags(work):
                 work["tags"] = [tag["name"] for tag in work["tags"]]
 
-        url_sanity = ("https://s.pximg.net/common/images"
-                      "/limit_sanity_level_360.png")
         ratings = {0: "General", 1: "R-18", 2: "R-18G"}
-        meta_user = self.config("metadata")
-        meta_bookmark = self.config("metadata-bookmark")
         metadata = self.metadata()
 
         works = self.works()
@@ -60,18 +66,26 @@ class PixivExtractor(Extractor):
             if not work["user"]["id"]:
                 continue
 
-            meta_single_page = work["meta_single_page"]
-            meta_pages = work["meta_pages"]
-            del work["meta_single_page"]
-            del work["image_urls"]
-            del work["meta_pages"]
+            files = self._extract_files(work)
 
-            if meta_user:
+            if self.meta_user:
                 work.update(self.api.user_detail(work["user"]["id"]))
-            if meta_bookmark and work["is_bookmarked"]:
+            if self.meta_comments:
+                if work["total_comments"]:
+                    work["comments"] = list(
+                        self.api.illust_comments(work["id"]))
+                else:
+                    work["comments"] = ()
+            if self.meta_bookmark and work["is_bookmarked"]:
                 detail = self.api.illust_bookmark_detail(work["id"])
                 work["tags_bookmark"] = [tag["name"] for tag in detail["tags"]
                                          if tag["is_registered"]]
+            if self.meta_captions and not work.get("caption") and \
+                    not work.get("_mypixiv") and not work.get("_ajax"):
+                body = self._request_ajax("/illust/" + str(work["id"]))
+                if body:
+                    work["caption"] = text.unescape(body["illustComment"])
+
             if transform_tags:
                 transform_tags(work)
             work["num"] = 0
@@ -81,69 +95,214 @@ class PixivExtractor(Extractor):
             work.update(metadata)
 
             yield Message.Directory, work
-
-            if work["type"] == "ugoira":
-                if not self.load_ugoira:
-                    continue
-
-                try:
-                    ugoira = self.api.ugoira_metadata(work["id"])
-                except exception.StopExtraction as exc:
-                    self.log.warning(
-                        "Unable to retrieve Ugoira metatdata (%s - %s)",
-                        work.get("id"), exc.message)
-                    continue
-
-                url = ugoira["zip_urls"]["medium"]
-                work["frames"] = frames = ugoira["frames"]
-                work["date_url"] = self._date_from_url(url)
-                work["_http_adjust_extension"] = False
-
-                if self.load_ugoira == "original":
-                    base, sep, _ = url.rpartition("_ugoira")
-                    base = base.replace(
-                        "/img-zip-ugoira/", "/img-original/", 1) + sep
-
-                    for ext in ("jpg", "png", "gif"):
-                        try:
-                            url = ("{}0.{}".format(base, ext))
-                            self.request(url, method="HEAD")
-                            break
-                        except exception.HttpError:
-                            pass
-                    else:
-                        self.log.warning(
-                            "Unable to find Ugoira frame URLs (%s)",
-                            work.get("id"))
-                        continue
-
-                    for num, frame in enumerate(frames):
-                        url = ("{}{}.{}".format(base, num, ext))
-                        work["num"] = work["_ugoira_frame_index"] = num
-                        work["suffix"] = "_p{:02}".format(num)
-                        text.nameext_from_url(url, work)
-                        yield Message.Url, url, work
-
-                else:
-                    url = url.replace("_ugoira600x600", "_ugoira1920x1080")
-                    yield Message.Url, url, text.nameext_from_url(url, work)
-
-            elif work["page_count"] == 1:
-                url = meta_single_page["original_image_url"]
-                if url == url_sanity:
-                    self.log.warning(
-                        "Unable to download work %s ('sanity_level' warning)",
-                        work["id"])
-                    continue
+            for work["num"], file in enumerate(files):
+                url = file["url"]
+                work.update(file)
                 work["date_url"] = self._date_from_url(url)
                 yield Message.Url, url, text.nameext_from_url(url, work)
 
+    def _extract_files(self, work):
+        meta_single_page = work["meta_single_page"]
+        meta_pages = work["meta_pages"]
+        del work["meta_single_page"]
+        del work["image_urls"]
+        del work["meta_pages"]
+
+        if meta_pages:
+            return [
+                {
+                    "url"   : img["image_urls"]["original"],
+                    "suffix": "_p{:02}".format(num),
+                    "_fallback": self._fallback_image(img),
+                }
+                for num, img in enumerate(meta_pages)
+            ]
+
+        url = meta_single_page["original_image_url"]
+        if url.startswith(self.limit_url):
+            work_id = work["id"]
+            self.log.debug("%s: %s", work_id, url)
+
+            limit_type = url.rpartition("/")[2]
+            if limit_type in (
+                "limit_",  # for '_extend_sanity()' inserts
+                "limit_unviewable_360.png",
+                "limit_sanity_level_360.png",
+            ):
+                work["_ajax"] = True
+                self.log.warning("%s: 'limit_sanity_level' warning", work_id)
+                if self.sanity_workaround:
+                    body = self._request_ajax("/illust/" + str(work_id))
+                    return self._extract_ajax(work, body)
+
+            elif limit_type == "limit_mypixiv_360.png":
+                work["_mypixiv"] = True
+                self.log.warning("%s: 'My pixiv' locked", work_id)
+
             else:
-                for work["num"], img in enumerate(meta_pages):
-                    url = img["image_urls"]["original"]
-                    work["date_url"] = self._date_from_url(url)
-                    work["suffix"] = "_p{:02}".format(work["num"])
-                    yield Message.Url, url, text.nameext_from_url(url, work)
+                work["_mypixiv"] = True  # stop further processing
+                self.log.error("%s: Unknown 'limit' URL type: %s",
+                               work_id, limit_type)
+
+        elif work["type"] != "ugoira":
+            return ({"url": url, "_fallback": self._fallback_image(url)},)
+
+        elif self.load_ugoira:
+            try:
+                return self._extract_ugoira(work, url)
+            except Exception as exc:
+                self.log.warning(
+                    "%s: Unable to retrieve Ugoira metatdata (%s - %s)",
+                    work["id"], exc.__class__.__name__, exc)
+
+        return ()
+
+    def _extract_ugoira(self, work, img_url):
+        ugoira = self.api.ugoira_metadata(work["id"])
+        work["_ugoira_frame_data"] = work["frames"] = frames = ugoira["frames"]
+        work["_ugoira_original"] = self.load_ugoira_original
+        work["_http_adjust_extension"] = False
+
+        if self.load_ugoira_original:
+            work["date_url"] = self._date_from_url(img_url)
+
+            base, sep, ext = img_url.rpartition("_ugoira0.")
+            if sep:
+                base += "_ugoira"
+            else:
+                base, sep, _ = img_url.rpartition("_ugoira")
+                base = base.replace(
+                    "/img-zip-ugoira/", "/img-original/", 1) + sep
+
+                for ext in ("jpg", "png", "gif"):
+                    try:
+                        url = "{}0.{}".format(base, ext)
+                        self.request(url, method="HEAD")
+                        break
+                    except exception.HttpError:
+                        pass
+                else:
+                    self.log.warning(
+                        "Unable to find Ugoira frame URLs (%s)", work["id"])
+
+            return [
+                {
+                    "url": "{}{}.{}".format(base, num, ext),
+                    "suffix": "_p{:02}".format(num),
+                    "_ugoira_frame_index": num,
+                }
+                for num in range(len(frames))
+            ]
+
+        else:
+            zip_url = ugoira["zip_urls"]["medium"]
+            work["date_url"] = self._date_from_url(zip_url)
+            url = zip_url.replace("_ugoira600x600", "_ugoira1920x1080", 1)
+            return ({"url": url},)
+
+    def _request_ajax(self, endpoint):
+        url = "{}/ajax{}".format(self.root, endpoint)
+        try:
+            return self.request(url, headers=self.headers_web).json()["body"]
+        except Exception:
+            return None
+
+    def _extract_ajax(self, work, body):
+        work["_ajax"] = True
+        url = self._extract_ajax_url(body)
+        if not url:
+            return ()
+
+        for key_app, key_ajax in (
+            ("title"            , "illustTitle"),
+            ("image_urls"       , "urls"),
+            ("create_date"      , "createDate"),
+            ("width"            , "width"),
+            ("height"           , "height"),
+            ("sanity_level"     , "sl"),
+            ("total_view"       , "viewCount"),
+            ("total_comments"   , "commentCount"),
+            ("total_bookmarks"  , "bookmarkCount"),
+            ("restrict"         , "restrict"),
+            ("x_restrict"       , "xRestrict"),
+            ("illust_ai_type"   , "aiType"),
+            ("illust_book_style", "bookStyle"),
+        ):
+            work[key_app] = body[key_ajax]
+
+        work["user"] = {
+            "account"    : body["userAccount"],
+            "id"         : int(body["userId"]),
+            "is_followed": False,
+            "name"       : body["userName"],
+            "profile_image_urls": {},
+        }
+
+        work["tags"] = tags = []
+        for tag in body["tags"]["tags"]:
+            name = tag["tag"]
+            try:
+                translated_name = tag["translation"]["en"]
+            except Exception:
+                translated_name = None
+            tags.append({"name": name, "translated_name": translated_name})
+
+        work["caption"] = text.unescape(body["illustComment"])
+        work["page_count"] = count = body["pageCount"]
+        if count == 1:
+            return ({"url": url},)
+
+        base, _, ext = url.rpartition("_p0.")
+        return [
+            {
+                "url"   : "{}_p{}.{}".format(base, num, ext),
+                "suffix": "_p{:02}".format(num),
+            }
+            for num in range(count)
+        ]
+
+    def _extract_ajax_url(self, body):
+        try:
+            original = body["urls"]["original"]
+            if original:
+                return original
+        except Exception:
+            pass
+
+        try:
+            square1200 = body["userIllusts"][body["id"]]["url"]
+        except Exception:
+            return
+        parts = square1200.rpartition("_p0")[0].split("/")
+        del parts[3:5]
+        parts[3] = "img-original"
+        base = "/".join(parts)
+
+        for ext in ("jpg", "png", "gif"):
+            try:
+                url = "{}_p0.{}".format(base, ext)
+                self.request(url, method="HEAD")
+                return url
+            except exception.HttpError:
+                pass
+
+    def _fallback_image(self, src):
+        if isinstance(src, str):
+            urls = None
+            orig = src
+        else:
+            urls = src["image_urls"]
+            orig = urls["original"]
+
+        base = orig.rpartition(".")[0]
+        yield base.replace("-original/", "-master/", 1) + "_master1200.jpg"
+
+        if urls is None:
+            return
+
+        for fmt in ("large", "medium", "square_medium"):
+            if fmt in urls:
+                yield urls[fmt]
 
     @staticmethod
     def _date_from_url(url, offset=timedelta(hours=9)):
@@ -218,10 +377,12 @@ class PixivArtworksExtractor(PixivExtractor):
                r"(?:/([^/?#]+))?/?(?:$|[?#])"
                r"|member_illust\.php\?id=(\d+)(?:&([^#]+))?)")
     example = "https://www.pixiv.net/en/users/12345/artworks"
+    _warn_phpsessid = True
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        u1, t1, u2, t2 = match.groups()
+    def _init(self):
+        PixivExtractor._init(self)
+
+        u1, t1, u2, t2 = self.groups
         if t1:
             t1 = text.unquote(t1)
         elif t2:
@@ -229,6 +390,15 @@ class PixivArtworksExtractor(PixivExtractor):
         self.user_id = u1 or u2
         self.tag = t1 or t2
 
+        if self.sanity_workaround:
+            self.cookies_domain = domain = ".pixiv.net"
+            self._init_cookies()
+            if self._warn_phpsessid:
+                PixivArtworksExtractor._warn_phpsessid = False
+                if not self.cookies.get("PHPSESSID", domain=domain):
+                    self.log.warning("No 'PHPSESSID' cookie set. Can detect on"
+                                     "ly non R-18 'limit_sanity_level' works.")
+
     def metadata(self):
         if self.config("metadata"):
             self.api.user_detail(self.user_id)
@@ -237,6 +407,20 @@ class PixivArtworksExtractor(PixivExtractor):
     def works(self):
         works = self.api.user_illusts(self.user_id)
 
+        if self.sanity_workaround:
+            body = self._request_ajax(
+                "/user/{}/profile/all".format(self.user_id))
+            try:
+                ajax_ids = list(map(int, body["illusts"]))
+                ajax_ids.extend(map(int, body["manga"]))
+                ajax_ids.sort()
+            except Exception as exc:
+                self.log.warning("u%s: Failed to collect artwork IDs "
+                                 "using AJAX API (%s: %s)",
+                                 self.user_id, exc.__class__.__name__, exc)
+            else:
+                works = self._extend_sanity(works, ajax_ids)
+
         if self.tag:
             tag = self.tag.lower()
             works = (
@@ -246,6 +430,35 @@ class PixivArtworksExtractor(PixivExtractor):
 
         return works
 
+    def _extend_sanity(self, works, ajax_ids):
+        user = {"id": 1}
+        index = len(ajax_ids) - 1
+
+        for work in works:
+            while index >= 0:
+                work_id = work["id"]
+                ajax_id = ajax_ids[index]
+
+                if ajax_id == work_id:
+                    index -= 1
+                    break
+
+                elif ajax_id > work_id:
+                    index -= 1
+                    self.log.debug("Inserting work %s", ajax_id)
+                    yield self._make_work(ajax_id, self.limit_url, user)
+
+                else:  # ajax_id < work_id
+                    break
+
+            yield work
+
+        while index >= 0:
+            ajax_id = ajax_ids[index]
+            self.log.debug("Inserting work %s", ajax_id)
+            yield self._make_work(ajax_id, self.limit_url, user)
+            index -= 1
+
 
 class PixivAvatarExtractor(PixivExtractor):
     """Extractor for pixiv avatars"""
@@ -255,12 +468,12 @@ class PixivAvatarExtractor(PixivExtractor):
     pattern = USER_PATTERN + r"/avatar"
     example = "https://www.pixiv.net/en/users/12345/avatar"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.user_id = match.group(1)
+    def _init(self):
+        PixivExtractor._init(self)
+        self.sanity_workaround = self.meta_comments = False
 
     def works(self):
-        user = self.api.user_detail(self.user_id)["user"]
+        user = self.api.user_detail(self.groups[0])["user"]
         url = user["profile_image_urls"]["medium"].replace("_170.", ".")
         return (self._make_work("avatar", url, user),)
 
@@ -273,12 +486,12 @@ class PixivBackgroundExtractor(PixivExtractor):
     pattern = USER_PATTERN + "/background"
     example = "https://www.pixiv.net/en/users/12345/background"
 
-    def __init__(self, match):
-        PixivExtractor.__init__(self, match)
-        self.user_id = match.group(1)
+    def _init(self):
+        PixivExtractor._init(self)
+        self.sanity_workaround = self.meta_comments = False
 
     def works(self):
-        detail = self.api.user_detail(self.user_id)
+        detail = self.api.user_detail(self.groups[0])
         url = detail["profile"]["background_image_url"]
         if not url:
             return ()
@@ -335,6 +548,22 @@ class PixivWorkExtractor(PixivExtractor):
         return works
 
 
+class PixivUnlistedExtractor(PixivExtractor):
+    """Extractor for a unlisted pixiv illustrations"""
+    subcategory = "unlisted"
+    pattern = BASE_PATTERN + r"/(?:en/)?artworks/unlisted/(\w+)"
+    example = "https://www.pixiv.net/en/artworks/unlisted/a1b2c3d4e5f6g7h8i9j0"
+
+    def _extract_files(self, work):
+        body = self._request_ajax("/illust/unlisted/" + work["id"])
+        work["id_unlisted"] = work["id"]
+        work["id"] = text.parse_int(body["illustId"])
+        return self._extract_ajax(work, body)
+
+    def works(self):
+        return ({"id": self.groups[0], "user": {"id": 1}},)
+
+
 class PixivFavoriteExtractor(PixivExtractor):
     """Extractor for all favorites/bookmarks of a pixiv user"""
     subcategory = "favorite"
@@ -413,7 +642,14 @@ class PixivRankingExtractor(PixivExtractor):
         self.mode = self.date = None
 
     def works(self):
-        return self.api.illust_ranking(self.mode, self.date)
+        ranking = self.ranking
+
+        works = self.api.illust_ranking(self.mode, self.date)
+        if self.type:
+            works = filter(lambda work, t=self.type: work["type"] == t, works)
+
+        for ranking["rank"], work in enumerate(works, 1):
+            yield work
 
     def metadata(self):
         query = text.parse_query(self.query)
@@ -452,10 +688,15 @@ class PixivRankingExtractor(PixivExtractor):
             date = (now - timedelta(days=1)).strftime("%Y-%m-%d")
         self.date = date
 
-        return {"ranking": {
+        self.type = type = query.get("content")
+
+        self.ranking = ranking = {
             "mode": mode,
             "date": self.date,
-        }}
+            "rank": 0,
+            "type": type or "all",
+        }
+        return {"ranking": ranking}
 
 
 class PixivSearchExtractor(PixivExtractor):
@@ -546,7 +787,6 @@ class PixivPixivisionExtractor(PixivExtractor):
     directory_fmt = ("{category}", "pixivision",
                      "{pixivision_id} {pixivision_title}")
     archive_fmt = "V{pixivision_id}_{id}{suffix}.{extension}"
-    cookies_domain = ".pixiv.net"
     pattern = r"(?:https?://)?(?:www\.)?pixivision\.net/(?:en/)?a/(\d+)"
     example = "https://www.pixivision.net/en/a/12345"
 
@@ -626,8 +866,6 @@ class PixivNovelExtractor(PixivExtractor):
                 work["tags"] = [tag["name"] for tag in work["tags"]]
 
         ratings = {0: "General", 1: "R-18", 2: "R-18G"}
-        meta_user = self.config("metadata")
-        meta_bookmark = self.config("metadata-bookmark")
         embeds = self.config("embeds")
         covers = self.config("covers")
 
@@ -645,9 +883,15 @@ class PixivNovelExtractor(PixivExtractor):
         if self.max_posts:
             novels = itertools.islice(novels, self.max_posts)
         for novel in novels:
-            if meta_user:
+            if self.meta_user:
                 novel.update(self.api.user_detail(novel["user"]["id"]))
-            if meta_bookmark and novel["is_bookmarked"]:
+            if self.meta_comments:
+                if novel["total_comments"]:
+                    novel["comments"] = list(
+                        self.api.novel_comments(novel["id"]))
+                else:
+                    novel["comments"] = ()
+            if self.meta_bookmark and novel["is_bookmarked"]:
                 detail = self.api.novel_bookmark_detail(novel["id"])
                 novel["tags_bookmark"] = [tag["name"] for tag in detail["tags"]
                                           if tag["is_registered"]]
@@ -848,6 +1092,7 @@ class PixivAppAPI():
         self.username = extractor._get_auth_info()[0]
         self.user = None
 
+        extractor.headers_web = extractor.session.headers.copy()
         extractor.session.headers.update({
             "App-OS"        : "ios",
             "App-OS-Version": "16.7.2",
@@ -913,6 +1158,10 @@ class PixivAppAPI():
         return self._call(
             "/v2/illust/bookmark/detail", params)["bookmark_detail"]
 
+    def illust_comments(self, illust_id):
+        params = {"illust_id": illust_id}
+        return self._pagination("/v3/illust/comments", params, "comments")
+
     def illust_follow(self, restrict="all"):
         params = {"restrict": restrict}
         return self._pagination("/v2/illust/follow", params)
@@ -935,6 +1184,10 @@ class PixivAppAPI():
         return self._call(
             "/v2/novel/bookmark/detail", params)["bookmark_detail"]
 
+    def novel_comments(self, novel_id):
+        params = {"novel_id": novel_id}
+        return self._pagination("/v1/novel/comments", params, "comments")
+
     def novel_detail(self, novel_id):
         params = {"novel_id": novel_id}
         return self._call("/v2/novel/detail", params)["novel"]
