@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2022 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -95,24 +95,7 @@ class ShopifyCollectionExtractor(ShopifyExtractor):
     subcategory = "collection"
     directory_fmt = ("{category}", "{collection[title]}")
     pattern = BASE_PATTERN + r"(/collections/[\w-]+)/?(?:$|[?#])"
-    test = (
-        ("https://chelseacrew.com/collections/flats"),
-        ("https://www.fashionnova.com/collections/mini-dresses", {
-            "range": "1-20",
-            "count": 20,
-        }),
-        ("https://www.fashionnova.com/collections/mini-dresses/?page=1"),
-        ("https://www.fashionnova.com/collections/mini-dresses#1"),
-        ("https://loungeunderwear.com/collections/apparel"),
-        ("https://michaels.com.au/collections/microphones"),
-        ("https://modcloth.com/collections/shoes"),
-        ("https://www.ohpolly.com/collections/dresses-mini-dresses"),
-        ("https://www.omgmiamiswimwear.com/collections/fajas"),
-        ("https://pinupgirlclothing.com/collections/evening"),
-        ("https://www.raidlondon.com/collections/flats"),
-        ("https://www.unique-vintage.com/collections/flapper-1920s"),
-        ("https://www.windsorstore.com/collections/dresses-ball-gowns"),
-    )
+    example = "https://www.fashionnova.com/collections/TITLE"
 
     def metadata(self):
         return self.request(self.item_url + ".json").json()
@@ -134,32 +117,7 @@ class ShopifyProductExtractor(ShopifyExtractor):
     subcategory = "product"
     directory_fmt = ("{category}", "Products")
     pattern = BASE_PATTERN + r"((?:/collections/[\w-]+)?/products/[\w-]+)"
-    test = (
-        ("https://chelseacrew.com/collections/flats/products/dora"),
-        ("https://www.fashionnova.com/products/essential-slide-red", {
-            "pattern": r"https?://cdn\d*\.shopify.com/",
-            "count": 3,
-        }),
-        ("https://www.fashionnova.com/collections/flats/products/name"),
-        ("https://de.loungeunderwear.com/products/ribbed-crop-top-black"),
-        ("https://michaels.com.au/collections/audio/products"
-         "/boya-by-wm4-pro-k5-2-4ghz-mic-android-1-1-101281"),
-        ("https://modcloth.com/collections/shoes/products/heidii-brn"),
-        (("https://www.ohpolly.com/products/edonia-ruched-triangle-cup"
-          "-a-line-mini-dress-brown")),
-        ("https://www.omgmiamiswimwear.com/products/la-medusa-maxi-dress", {
-            "pattern": r"https://cdn\.shopify\.com/s/files/1/1819/6171/",
-            "count": 5,
-        }),
-        ("https://pinupgirlclothing.com/collections/evening/products"
-         "/clarice-coat-dress-in-olive-green-poly-crepe-laura-byrnes-design"),
-        ("https://www.raidlondon.com/collections/flats/products"
-         "/raid-addyson-chunky-flat-shoe-in-white"),
-        ("https://www.unique-vintage.com/collections/flapper-1920s/products"
-         "/unique-vintage-plus-size-black-silver-beaded-troyes-flapper-dress"),
-        ("https://www.windsorstore.com/collections/accessories-belts/products"
-         "/rhine-buckle-dbl-o-ring-pu-strap-belt-073010158001"),
-    )
+    example = "https://www.fashionnova.com/collections/TITLE/products/NAME"
 
     def products(self):
         product = self.request(self.item_url + ".json").json()["product"]
