@@ -45,11 +45,15 @@ class TestDownloaderModule(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         # allow import of ytdl downloader module without youtube_dl installed
+        cls._orig_ytdl = sys.modules.get("youtube_dl")
         sys.modules["youtube_dl"] = MagicMock()
 
     @classmethod
     def tearDownClass(cls):
-        del sys.modules["youtube_dl"]
+        if cls._orig_ytdl:
+            sys.modules["youtube_dl"] = cls._orig_ytdl
+        else:
+            del sys.modules["youtube_dl"]
 
     def tearDown(self):
         downloader._cache.clear()
@@ -136,8 +140,8 @@ class TestDownloaderBase(unittest.TestCase):
 
         if content:
             mode = "w" + ("b" if isinstance(content, bytes) else "")
-            with pathfmt.open(mode) as file:
-                file.write(content)
+            with pathfmt.open(mode) as fp:
+                fp.write(content)
 
         return pathfmt
 
@@ -151,8 +155,8 @@ class TestDownloaderBase(unittest.TestCase):
 
         # test content
         mode = "r" + ("b" if isinstance(output, bytes) else "")
-        with pathfmt.open(mode) as file:
-            content = file.read()
+        with pathfmt.open(mode) as fp:
+            content = fp.read()
         self.assertEqual(content, output)
 
         # test filename extension
